package org.geek.szbay.assistant.code.generator;

import java.util.List;

import org.geek.szbay.assistant.code.generator.helpper.TableHelpper;
import org.geek.szbay.assistant.utils.StringUtils;

public class MapperXmlMaker {

	public static final String parse(String authorName, String tableName, String tableComment, String key,
			String outPath, String entityFullClassName, String entityClassName, String daoFullClassName,
			String daoClassName, List<String> attrNames, List<String> colNames, List<String> colTypes,
			List<String> colComments) {
		StringBuffer sb = new StringBuffer();
		String className = TableHelpper.columnNameToAttrName(tableName);
		String keyAttr = TableHelpper.columnNameToAttrName(key);

		parseHead(sb);
		parseNamespace(sb, daoFullClassName);
		parseResultMap(sb, key, keyAttr, className, entityFullClassName, colNames, attrNames);
		parseAllField(sb, colNames);

		parseInsert(sb, key, keyAttr, tableName, entityFullClassName, colNames, attrNames, colTypes);

		parseUpdate(sb, key, keyAttr, tableName, entityFullClassName, colNames, attrNames, colTypes);
		parseUpdateByExample(sb, key, keyAttr, tableName, entityFullClassName, colNames, attrNames, colTypes);

		parseDelete(sb, key, keyAttr, tableName, entityFullClassName, colNames, attrNames);
		parseDeleteByKey(sb, key, keyAttr, tableName, entityFullClassName, colNames, attrNames);
		parseDeleteByExample(sb, key, keyAttr, tableName, entityFullClassName, colNames, attrNames, colTypes);

		parseCountByExample(sb, key, keyAttr, className, tableName, colNames, attrNames, colTypes);
		parseListByExample(sb, key, keyAttr, className, tableName, colNames, attrNames, colTypes);

		parseGetByKey(sb, key, keyAttr, className, tableName);

		parseEnd(sb);

		return sb.toString();
	}

	private static final void parseHead(StringBuffer sb) {
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>").append("\r\n");
		sb.append(
				"<!DOCTYPE mapper PUBLIC \"-//mybatis.org//DTD Mapper 3.0//EN\" \"http://mybatis.org/dtd/mybatis-3-mapper.dtd\">")
				.append("\r\n");
	}

	private static final void parseNamespace(StringBuffer sb, String readFullClassName) {
		sb.append("<mapper namespace=\"").append(readFullClassName).append("\"> ").append("\r\n");
		sb.append("\r\n");
	}

	private static final void parseResultMap(StringBuffer sb, String key, String keyAttr, String className,
			String entityFullClassName, List<String> colNames, List<String> attrNames) {
		sb.append("    <resultMap ").append("id=\"").append(className).append("Map\" type=\"")
				.append(entityFullClassName).append("\">").append("\r\n");
		sb.append("        <id column=\"").append(key).append("\" property=\"").append(keyAttr).append("\"/>")
				.append("\r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			if (column.equals(key)) {
				continue;
			}

			String attr = attrNames.get(i);

			sb.append("        <result column=\"").append(column).append("\" property=\"").append(attr).append("\"/>")
					.append("\r\n");
		}

		sb.append("    </resultMap>\r\n");
		sb.append("\r\n");
	}

	private static final void parseAllField(StringBuffer sb, List<String> colNames) {
		sb.append("    <sql id=\"allField\">").append("\r\n");
		sb.append("        ");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			sb.append(column);
			if (i != colNames.size() - 1) {
				sb.append(",");
			}
		}

		sb.append("\r\n");
		sb.append("    </sql>\r\n");
		sb.append("\r\n");
	}

	private static final void parseDelete(StringBuffer sb, String key, String keyAttr, String tableName,
			String entityFullClassName, List<String> colNames, List<String> attrNames) {
		sb.append("    <delete id=\"delete\">\r\n");
		sb.append("        delete from " + tableName + "\r\n");
		sb.append("        where " + key + " = #{key}\r\n");
		sb.append("    </delete>\r\n");
		sb.append("\r\n");
	}

	private static final void parseDeleteByKey(StringBuffer sb, String key, String keyAttr, String tableName,
			String entityFullClassName, List<String> colNames, List<String> attrNames) {
		sb.append("    <delete id=\"deleteByKey\">\r\n");
		sb.append("        delete from " + tableName + "\r\n");
		sb.append("        where " + key + " = #{delete." + keyAttr + "}\r\n");
		sb.append("    </delete>\r\n");
		sb.append("\r\n");
	}

	private static void parseDeleteByExample(StringBuffer sb, String key, String keyAttr, String tableName,
			String entityFullClassName, List<String> colNames, List<String> attrNames, List<String> colTypes) {
		sb.append("    <delete id=\"deleteByExample\"> \r\n");
		sb.append("        delete from " + tableName + "\r\n");
		sb.append("        where 1 = 1 \r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			String attr = attrNames.get(i);

			sb.append("        <if test=\"example.").append(attr).append(" != null\" >\r\n");
			sb.append("            and ").append(column).append(" = #{example.").append(attr).append("} \r\n");
			sb.append("        </if>\r\n");
		}

		sb.append("    </delete>\r\n");
		sb.append("\r\n");
	}

	private static final void parseInsert(StringBuffer sb, String key, String keyAttr, String tableName,
			String entityFullClassName, List<String> colNames, List<String> attrNames, List<String> colTypes) {
		sb.append("    <insert id=\"save\" useGeneratedKeys=\"");
		if ("id".equals(key)) {
			sb.append("true\" ");
		} else {
			sb.append("false\" ");
		}
		sb.append("keyProperty=\"").append(keyAttr).append("\" ");
		sb.append("parameterType=\"").append(entityFullClassName).append("\">\r\n");

		sb.append("        insert into ").append(tableName).append("\r\n");
		sb.append("        <trim prefix=\"(\" suffix=\")\" suffixOverrides=\",\">\r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			String attr = attrNames.get(i);
			boolean typeIsString = TableHelpper.colTypeIsString(colTypes.get(i));

			if (column.equals(key) && "id".equals(key)) {
				// key自动生成，所以不做任何处理
				continue;
			}

			if (typeIsString) {
				sb.append("            <if test=\"").append(attr).append(" != null").append(" and ").append(attr)
						.append(" != ''\" >\r\n");
			} else {
				sb.append("            <if test=\"").append(attr).append(" != null\" >\r\n");
			}

			sb.append("                ").append(column).append(", \r\n");
			sb.append("            </if>\r\n");
		}

		sb.append("        </trim>\r\n");

		sb.append("        <trim prefix=\"values (\" suffix=\")\" suffixOverrides=\",\">\r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String attr = attrNames.get(i);
			boolean typeIsString = TableHelpper.colTypeIsString(colTypes.get(i));

			if (attr.equals("id")) {
				continue;
			}

			if (typeIsString) {
				sb.append("            <if test=\"").append(attr).append(" != null").append(" and ").append(attr)
						.append(" != ''\" >\r\n");
			} else {
				sb.append("            <if test=\"").append(attr).append(" != null\" >\r\n");
			}

			sb.append("                #{").append(attr).append("}, \r\n");
			sb.append("            </if>\r\n");
		}

		sb.append("        </trim>\r\n");

		sb.append("    </insert>\r\n");
		sb.append("\r\n");
	}

	private static final void parseUpdate(StringBuffer sb, String key, String keyAttr, String tableName,
			String entityFullClassName, List<String> colNames, List<String> attrNames, List<String> colTypes) {
		sb.append("    <update id=\"update\"");
		sb.append(" parameterType=\"").append(entityFullClassName).append("\">\r\n");

		sb.append("        update ").append(tableName).append("\r\n");
		sb.append("        <set>\r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			String attr = attrNames.get(i);

			if (column.equals(key)) {
				// key不做任何处理
				continue;
			}

			sb.append("            <if test=\"").append(attr).append(" != null\" >\r\n");

			sb.append("                ").append(column).append(" = #{").append(attr).append("}, \r\n");
			sb.append("            </if>\r\n");
		}

		sb.append("        </set>\r\n");

		sb.append("        where ").append(key).append(" = #{").append(keyAttr).append("}\r\n");

		sb.append("    </update>\r\n");
		sb.append("\r\n");
	}

	private static final void parseGetByKey(StringBuffer sb, String key, String keyAttr, String className,
			String tableName) {
		sb.append("    <select id=\"getByKey\" ").append("resultMap=\"").append(StringUtils.firstToLowerCase(className))
				.append("Map\">").append("\r\n");
		sb.append("        select <include refid=\"allField\"/>\r\n");
		sb.append("        from ").append(tableName).append(" where ").append(key).append(" = #{key}\r\n");
		sb.append("    </select>\r\n");
		sb.append("\r\n");
	}

	private static void parseCountByExample(StringBuffer sb, String key, String keyAttr, String className,
			String tableName, List<String> colNames, List<String> attrNames, List<String> colTypes) {
		sb.append("    <select id=\"countByExample\"").append(" resultType=\"java.lang.Integer\"").append("> \r\n");
		sb.append("        select count(*) ").append("from ").append(tableName).append(" where 1 = 1 ").append("\r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			String attr = attrNames.get(i);
			boolean typeIsString = TableHelpper.colTypeIsString(colTypes.get(i));

			if (typeIsString) {
				sb.append("        <if test=\"example.").append(attr).append(" != null").append(" and example.")
						.append(attr).append(" != ''\" >\r\n");
			} else {
				sb.append("        <if test=\"example.").append(attr).append(" != null\" >\r\n");
			}

			sb.append("            and ").append(column).append(" = #{example.").append(attr).append("} \r\n");
			sb.append("        </if>\r\n");
		}

		sb.append("    </select>\r\n");
		sb.append("\r\n");
	}

	private static void parseUpdateByExample(StringBuffer sb, String key, String keyAttr, String tableName,
			String entityFullClassName, List<String> colNames, List<String> attrNames, List<String> colTypes) {
		sb.append("    <update id=\"updateByExample\">\r\n");
		sb.append("        update ").append(tableName).append("\r\n");
		sb.append("        <set>\r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			String attr = attrNames.get(i);

			if (column.equals(key)) {
				// key不做任何处理
				continue;
			}

			sb.append("            <if test=\"update.").append(attr).append(" != null\" >\r\n");
			sb.append("                ").append(column).append(" = #{update.").append(attr).append("}, \r\n");
			sb.append("            </if>\r\n");
		}

		sb.append("        </set>\r\n");
		sb.append("        where 1 = 1 \r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			String attr = attrNames.get(i);

			sb.append("        <if test=\"example.").append(attr).append(" != null\" >\r\n");
			sb.append("            and ").append(column).append(" = #{example.").append(attr).append("} \r\n");
			sb.append("        </if>\r\n");
		}

		sb.append("    </update>\r\n");
		sb.append("\r\n");
	}

	private static final void parseListByExample(StringBuffer sb, String key, String keyAttr, String className,
			String tableName, List<String> colNames, List<String> attrNames, List<String> colTypes) {
		sb.append("    <select id=\"listByExample\" ").append(" resultMap=\"")
				.append(StringUtils.firstToLowerCase(className)).append("Map\">").append("\r\n");
		sb.append("        select <include refid=\"allField\" />\r\n");
		sb.append("        from ").append(tableName).append(" where 1 = 1 ").append("\r\n");

		for (int i = 0; i < colNames.size(); i++) {
			String column = colNames.get(i);
			String attr = attrNames.get(i);
			boolean typeIsString = TableHelpper.colTypeIsString(colTypes.get(i));

			if (typeIsString) {
				sb.append("        <if test=\"example.").append(attr).append(" != null").append(" and example.")
						.append(attr).append(" != ''\" >\r\n");
			} else {
				sb.append("        <if test=\"example.").append(attr).append(" != null\" >\r\n");
			}

			sb.append("            and ").append(column).append(" = #{example.").append(attr).append("} \r\n");
			sb.append("        </if>\r\n");
		}

		sb.append("        <if test=\"orderBy != null\" >\r\n");
		sb.append("            order by ${orderBy} \r\n");
		sb.append("        </if>\r\n");
		sb.append("        limit ${pageSize} offset ${(page - 1) * pageSize}").append("\r\n");

		sb.append("    </select>\r\n");
		sb.append("\r\n");
	}

	private static final void parseEnd(StringBuffer sb) {
		sb.append("\r\n");
		sb.append("</mapper>");
		sb.append("\r\n");
	}

}
